(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2013-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uordered_field_spec.mli,v 1.7 2014-01-06 13:53:08 fclement Exp $ *)

module type STRUCTURE =
sig
  type t;;
  
  val _0_ : t;;
  val _1_ : t;;
  
  val is_zero : t -> t -> bool;;
  
  val abs : t -> t;;
  
  val ( +. ) : t -> t -> t;;
  val ( ~-. ) : t -> t;; (* Opposite. *)
  val ( *. ) : t -> t -> t;;
  val ( ~/. ) : t -> t;; (* Inverse. *)
  
  val of_float : float -> t;;
  val to_float : t -> float;;
  
  val of_int : int -> t;;
  val to_int : t -> int;;
  
  val of_bit_string : string -> t;;
  val to_bit_string : t -> string;;
  
  val print : t Format_printer.printer;;
  val printf : out_channel -> t -> unit;;
  
  val compare : t -> t -> int;;
end
;;

module type ORDERED_FIELD =
sig
  include STRUCTURE
  
  val _2_ : t;;
  val _3_ : t;;
  val _4_ : t;;
  val _5_ : t;;
  val _6_ : t;;
  val _7_ : t;;
  val _8_ : t;;
  val _9_ : t;;
  val minus_1_ : t;;
  val minus_2_ : t;;
  val minus_3_ : t;;
  val minus_4_ : t;;
  val minus_5_ : t;;
  val minus_6_ : t;;
  val minus_7_ : t;;
  val minus_8_ : t;;
  val minus_9_ : t;;
  val inv_2_ : t;;
  val inv_3_ : t;;
  val inv_4_ : t;;
  val inv_5_ : t;;
  val inv_6_ : t;;
  val inv_7_ : t;;
  val inv_8_ : t;;
  val inv_9_ : t;;
  
  val ( -. ) : t -> t -> t;;
  val ( /. ) : t -> t -> t;;
  val axpy : t -> t -> t -> t;;
  val mean : t -> t -> t;;
  val square : t -> t;;
  
  val of_nat : Unatural.t -> t;;
  val to_nat : t -> Unatural.t;;
end
;;

module type MAKER = functor (S : STRUCTURE) -> ORDERED_FIELD;;
